"use strict";
/* --------------------------------------------------------------------------------------------
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Symbols = exports.Type = exports.LanguageServiceHost = exports.CompileOptions = void 0;
const path = require("path");
const crypto = require("crypto");
const ts = require("typescript");
const isWindows = process.platform === 'win32';
function normalizePath(value) {
    if (isWindows) {
        value = value.replace(/\\/g, '/');
        if (/^[a-z]:/.test(value)) {
            value = value.charAt(0).toUpperCase() + value.substring(1);
        }
    }
    const result = path.posix.normalize(value);
    return result.length > 0 && result.charAt(result.length - 1) === '/' ? result.substr(0, result.length - 1) : result;
}
function makeAbsolute(p, root) {
    if (path.isAbsolute(p)) {
        return normalizePath(p);
    }
    if (root === undefined) {
        return normalizePath(path.join(process.cwd(), p));
    }
    else {
        return normalizePath(path.join(root, p));
    }
}
var CompileOptions;
(function (CompileOptions) {
    function getConfigFilePath(options) {
        if (options.project) {
            const projectPath = path.resolve(options.project);
            if (ts.sys.directoryExists(projectPath)) {
                return normalizePath(path.join(projectPath, 'tsconfig.json'));
            }
            else {
                return normalizePath(projectPath);
            }
        }
        const result = options.configFilePath;
        return result && makeAbsolute(result);
    }
    CompileOptions.getConfigFilePath = getConfigFilePath;
    function getDefaultOptions(configFileName) {
        const options = configFileName && path.basename(configFileName) === 'jsconfig.json'
            ? { allowJs: true, maxNodeModuleJsDepth: 2, allowSyntheticDefaultImports: true, skipLibCheck: true, noEmit: true }
            : {};
        return options;
    }
    CompileOptions.getDefaultOptions = getDefaultOptions;
})(CompileOptions = exports.CompileOptions || (exports.CompileOptions = {}));
var LanguageServiceHost;
(function (LanguageServiceHost) {
    function useSourceOfProjectReferenceRedirect(host, value) {
        host.useSourceOfProjectReferenceRedirect = value;
    }
    LanguageServiceHost.useSourceOfProjectReferenceRedirect = useSourceOfProjectReferenceRedirect;
})(LanguageServiceHost = exports.LanguageServiceHost || (exports.LanguageServiceHost = {}));
var Type;
(function (Type) {
    function isObjectType(type) {
        return (type.flags & ts.TypeFlags.Object) !== 0;
    }
    Type.isObjectType = isObjectType;
    function isTypeReference(type) {
        return (type.objectFlags & ts.ObjectFlags.Reference) !== 0;
    }
    Type.isTypeReference = isTypeReference;
    function isVoidType(type) {
        return (type.flags & ts.TypeFlags.Void) !== 0;
    }
    Type.isVoidType = isVoidType;
    function isNullType(type) {
        return (type.flags & ts.TypeFlags.Null) !== 0;
    }
    Type.isNullType = isNullType;
    function isUndefinedType(type) {
        return (type.flags & ts.TypeFlags.Undefined) !== 0;
    }
    Type.isUndefinedType = isUndefinedType;
})(Type = exports.Type || (exports.Type = {}));
class Symbols {
    constructor(typeChecker) {
        this.typeChecker = typeChecker;
    }
    static isClass(symbol) {
        return symbol !== undefined && (symbol.getFlags() & ts.SymbolFlags.Class) !== 0;
    }
    static isInterface(symbol) {
        return symbol !== undefined && (symbol.getFlags() & ts.SymbolFlags.Interface) !== 0;
    }
    static isTypeLiteral(symbol) {
        return symbol !== undefined && (symbol.getFlags() & ts.SymbolFlags.TypeLiteral) !== 0;
    }
    static isAliasSymbol(symbol) {
        return symbol !== undefined && (symbol.getFlags() & ts.SymbolFlags.Alias) !== 0;
    }
    static isTypeAlias(symbol) {
        return symbol !== undefined && (symbol.getFlags() & ts.SymbolFlags.TypeAlias) !== 0;
    }
    static isPrototype(symbol) {
        return symbol !== undefined && (symbol.getFlags() & ts.SymbolFlags.Prototype) !== 0;
    }
    static isRegularEnum(symbol) {
        return symbol !== undefined && (symbol.getFlags() & ts.SymbolFlags.RegularEnum) !== 0;
    }
    static isProperty(symbol) {
        return symbol !== undefined && (symbol.getFlags() & ts.SymbolFlags.Property) !== 0;
    }
    static isOptional(symbol) {
        return symbol !== undefined && (symbol.getFlags() & ts.SymbolFlags.Optional) !== 0;
    }
    static getParent(symbol) {
        return symbol.parent;
    }
    createKey(symbol) {
        let result = symbol.__symbol__data__key__;
        if (result !== undefined) {
            return result;
        }
        let declarations = symbol.getDeclarations();
        if (declarations === undefined) {
            if (this.typeChecker.isUnknownSymbol(symbol)) {
                return Symbols.Unknown;
            }
            else if (this.typeChecker.isUndefinedSymbol(symbol)) {
                return Symbols.Undefined;
            }
            else {
                return Symbols.None;
            }
        }
        let fragments = [];
        for (let declaration of declarations) {
            fragments.push({
                f: declaration.getSourceFile().fileName,
                s: declaration.getStart(),
                e: declaration.getEnd(),
                k: declaration.kind
            });
        }
        if (fragments.length > 1) {
            fragments.sort((a, b) => {
                let result = a.f < b.f ? -1 : (a.f > b.f ? 1 : 0);
                if (result !== 0) {
                    return result;
                }
                result = a.s - b.s;
                if (result !== 0) {
                    return result;
                }
                result = a.e - b.e;
                if (result !== 0) {
                    return result;
                }
                return a.k - b.k;
            });
        }
        const hash = crypto.createHash('md5');
        if ((symbol.flags & ts.SymbolFlags.Transient) !== 0) {
            hash.update(JSON.stringify({ trans: true }, undefined, 0));
        }
        hash.update(JSON.stringify(fragments, undefined, 0));
        result = hash.digest('base64');
        symbol.__symbol__data__key__ = result;
        return result;
    }
    computeBaseSymbolsForClass(symbol) {
        const result = [];
        const declarations = symbol.getDeclarations();
        if (declarations === undefined) {
            return undefined;
        }
        const typeChecker = this.typeChecker;
        for (const declaration of declarations) {
            if (ts.isClassDeclaration(declaration)) {
                const heritageClauses = declaration.heritageClauses;
                if (heritageClauses) {
                    for (const heritageClause of heritageClauses) {
                        for (const type of heritageClause.types) {
                            const tsType = typeChecker.getTypeAtLocation(type.expression);
                            if (tsType !== undefined) {
                                const baseSymbol = tsType.getSymbol();
                                if (baseSymbol !== undefined && baseSymbol !== symbol) {
                                    result.push(baseSymbol);
                                }
                            }
                        }
                    }
                }
            }
        }
        return result.length === 0 ? undefined : result;
    }
    computeBaseSymbolsForInterface(symbol) {
        const result = [];
        const tsType = this.typeChecker.getDeclaredTypeOfSymbol(symbol);
        if (tsType === undefined) {
            return undefined;
        }
        const baseTypes = tsType.getBaseTypes();
        if (baseTypes !== undefined) {
            for (let base of baseTypes) {
                let symbol = base.getSymbol();
                if (symbol) {
                    result.push(symbol);
                }
            }
        }
        return result.length === 0 ? undefined : result;
    }
    getTypeOfSymbol(symbol) {
        if (Symbols.isTypeAlias(symbol) || Symbols.isInterface(symbol)) {
            return this.typeChecker.getDeclaredTypeOfSymbol(symbol);
        }
        const location = this.inferLocationNode(symbol);
        if (location !== undefined) {
            return this.typeChecker.getTypeOfSymbolAtLocation(symbol, location);
        }
        else {
            return this.typeChecker.getDeclaredTypeOfSymbol(symbol);
        }
    }
    inferLocationNode(symbol) {
        const declarations = symbol.declarations;
        if (declarations !== undefined && declarations.length > 0) {
            return declarations[0];
        }
        if (Symbols.isPrototype(symbol)) {
            const parent = Symbols.getParent(symbol);
            if (parent !== undefined) {
                return this.inferLocationNode(parent);
            }
        }
        return undefined;
    }
}
exports.Symbols = Symbols;
Symbols.Unknown = 'unknown';
Symbols.Undefined = 'undefined';
Symbols.None = 'none';
//# sourceMappingURL=typescripts.js.map